# sb::File::Audio
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::File::Audio;
use strict;
use sb::File ();
use base qw( sb::File );
use vars qw( $VERSION );
$VERSION = '0.05';
# 0.05 [2007/07/16] changed return to make them consistent
# 0.04 [2006/07/04] got rid of "return( undef )"
# 0.03 [2006/03/17] changed bracket rule
# 0.02 [2006/03/09] changed _set_audio to fix a bug
# 0.01 [2006/02/22] added Sound::Info class
# 0.00 [2005/10/31] generated
# --------------------------------------
# // override base class
sub initialize
{
	my $self = shift;
	$self->SUPER::initialize(@_);
	return $self->_set_audio;
}
# --------------------------------------
# // public functions
sub is_audio
{
	my $self = shift;
	return 1 if ($self->content_type eq 'audio/mpeg');
	return 1 if ($self->content_type eq 'audio/x-m4a');
	return 1 if ($self->content_type eq 'audio/x-m4p');
	return 1 if ($self->content_type eq 'video/mp4');
	return;
}
sub audio_info
{
	my $self = shift;
	my $label = shift;
	return if (!$label);
	return if (!$self->is_audio);
	$self->_set_audio if (!$self->{'param'}->{'audio'});
	return if (!$self->{'param'}->{'audio'});
	return $self->{'param'}->{'audio'}->$label();
}
# --------------------------------------
# // private functions
sub _set_audio
{
	my $self = shift;
	return $self if (!$sb::File::Audio::Info::LOADED_LIB);
	return $self if (!$self->exists or !$self->is_audio);
	$self->{'param'}->{'audio'} = sb::File::Audio::Info->new($self->fullpath);
	return $self;
}
# ======================================
package sb::File::Audio::Info;
use sb::Object;
use base qw( sb::Object );
use vars qw( $VERSION $LOADED_LIB );
$VERSION = $sb::File::Audio::VERSION; # version is the same as sb::File::Audio
BEGIN
{
	eval {
		require MP3::Info;
		require MP4::Info;
	};
	$LOADED_LIB = ($@) ? undef : 1;
}
# --------------------------------------
# // override base class
sub initialize
{
	my $self = shift;
	my $file = shift;
	my %param = (
		'title'    => '',
		'duration' => '',
		'artwork'  => undef,
		'arttype'  => '', # jpg or png
	);
	$self->generate_accessor( keys(%param) );
	return $self->SUPER::initialize(%param)->_initialize($file);
}
# --------------------------------------
# // private functions
sub _initialize
{
	my $self = shift;
	my $file = shift;
	return $self if (!$LOADED_LIB);
	if ($file =~ /\.mp3$/)
	{
		my $tags = &MP3::Info::get_mp3tag($file,2,2); # FILE, VERSION, RAW_V2
		my $info = &MP3::Info::get_mp3info($file);
		$self->{'artwork'} = &_extract_image($tags->{'PIC'}) if ($tags->{'PIC'});
		$self->{'title'} = ($tags->{'TT2'}) ? $tags->{'TT2'} : $tags->{'TIT2'};
		$self->{'duration'} = $info->{'TIME'};
	}
	elsif ($file =~ /\.m4a$|\.m4p$|\.mp4$/)
	{
		my $mp4 = MP4::Info->new($file);
		$self->{'artwork'} = $mp4->covr;
		$self->{'title'} = $mp4->title;
		$self->{'duration'} = $mp4->time;
	}
	$self->{'arttype'} = &_check_arttype($self->{'artwork'}) if ($self->{'artwork'});
	return $self; # need to pass $self instance
}
sub _extract_image
{
	my $data = shift;
	# my $format = substr($data,0,3); # JPG or PNG
	# my $type = substr($data,3,1); # picture type
	$data = substr($data,5); # remove header
	return $data;
	# [reference] format of PIC and APIC in ID3tag
	# http://silhouette.studio.mu/artwork.shtml
}
sub _check_arttype
{
	my $pict = shift;
	my $head = substr($pict,0,10);
	return
		  ($head eq "\x89\x50\x4e\x47\x0d\x0a\x1a\x0a\x00\x00") ? 'png'
		: ($head eq "\xff\xd8\xff\xe0\x00\x10\x4a\x46\x49\x46") ? 'jpg'
		: ''; # except png and jpg
}
1;
__END__
