// spica javascript libraries - pickuplist.js
// == written by Takuya Otani <takuya.otani@gmail.com> ===
// == Copyright (C) 2006 SimpleBoxes/SerendipityNZ Ltd. ==

// [guideline]
// -- http://serennz.sakura.ne.jp/guidelines/js_en_000.txt
// -- http://serennz.sakura.ne.jp/guidelines/js_ja_000.txt

// ver 0.00 [2008/11/01] the first attempt

Spica.Library.require('element');
Spica.PickupList = function(label)
{
	if (!label) label = 'pikcuplist';
	this.label = label;
	this.list_of = {};
	this.ignored = [];
	return this;
};
Spica.PickupList.prototype = {
	bindForm : function(form)
	{
		var self = this;
		if (!form) return;
		Spica.Event.register(form,'submit',function() { self.onSubmit(); });
	},
	setupTitle : function(label_of)
	{
		var self = this;
		for (var name in label_of)
		{
			var obj = Spica.$(self.label + '_label_' + name);
			if (!obj) continue;
			obj.innerHTML = label_of[name];
		}
	},
	setupMover : function()
	{
		var self = this;
		Spica.Event.register(Spica.$(self.label + '_add'),'click',function() { self.onVoted(); });
		Spica.Event.register(Spica.$(self.label + '_sub'),'click',function() { self.onRejected(); });
	},
	createTable : function(target)
	{
		var self = this;
		if (!target) return;
		var table = document.createElement('table');
		var lead = document.createElement('tr');
		var list = document.createElement('tr');
		table.appendChild(lead);
		table.appendChild(list);
		var temp = '<th id="%%_label_candidate">candidate</th><td>&nbsp;</td><th id="%%_label_selected">selected</th>';
		lead.innerHTML = temp.replace(/%%/g,self.label);
		temp = '<td><select name="%%_candidate" multiple="multiple" id="%%_candidate"></select></td><td><p><input type="button" value="&gt;&gt;" id="%%_add" /></p><p><input type="button" value="&lt;&lt;" id="%%_sub" /></p></td><td><select name="%%_selected" multiple="multiple" id="%%_selected"></select></td>';
		list.innerHTML = temp.replace(/%%/g,self.label);
		target.appendChild(table);
	},
	onSubmit : function()
	{
		var self = this;
		var selected = Spica.$(self.label + '_selected');
		if (!selected) return;
		for (var i=0;i<selected.options.length;i++)
			selected.options[i].selected = true;
	},
	updateList : function()
	{
		var self = this;
		var candidate = Spica.$(self.label + '_candidate');
		var selected = Spica.$(self.label + '_selected');
		if (!candidate || !selected) return;
		candidate.options.length = 0;
		selected.options.length = 0;
		for (var id in self.list_of)
		{
			if (self.ignored.length > 0 && self.ignored.indexOf(id) > -1) continue;
			var opt = new Option;
			opt.value = id;
			opt.text = self.list_of[id].text;
			opt.setAttribute('title',self.list_of[id].text);
			if (self.list_of[id].selected)
			{
				var num = selected.options.length;
				selected.options[num] = opt;
			}
			else
			{
				var num = candidate.options.length;
				candidate.options[num] = opt;
			}
		}
	},
	setIgnored : function(list)
	{
		var self = this;
		self.ignored = []; // reset ignored list
		for (var i=0,n=list.length;i<n;i++)
			self.ignored.push(list[i]);
		self.updateList();
	},
	removeCandidate : function(id)
	{
		var self = this;
		if (!self.list_of[id]) return; // has been removed
		self.list_of[id] = null;
		self.updateList();
	},
	setCandidate : function(candidate)
	{
		var self = this;
		var selected = Spica.$(self.label + '_selected');
		if (!selected) return;
		self.list_of[candidate.value] = {
			selected: false,
			text: candidate.text
		};
		for (var i=0,n=selected.options.length;i<n;i++)
		{
			if (selected.options[i].value != candidate.value) continue;
			self.list_of[candidate.value].selected = true;
			break;
		}
	},
	onVoted : function()
	{
		var self = this;
		var list = Spica.$(self.label + '_candidate');
		for (var i=0,n=list.options.length;i<n;i++)
		{
			var opt = list.options[i];
			if (opt.selected && self.list_of[opt.value])
				self.list_of[opt.value].selected = true;
		}
		self.updateList();
	},
	onRejected : function()
	{
		var self = this;
		var list = Spica.$(self.label + '_selected');
		for (var i=0,n=list.options.length;i<n;i++)
		{
			var opt = list.options[i];
			if (opt.selected && self.list_of[opt.value])
				self.list_of[opt.value].selected = false;
		}
		self.updateList();
	}
};
